searchData={"items":[{"type":"module","title":"ParameterizedTest","doc":"A utility for defining eminently readable parameterized (or example-based) tests.\n\nParameterized tests look like this:\n\n    param_test \"grants free shipping based on the marketing site's stated policy\",\n               \"\"\"\n               | spending_by_category          | coupon      | ships_free? | description      |\n               |-------------------------------|-------------|-------------|------------------|\n               | %{shoes: 19_99, pants: 29_99} |             | false       | Spent too little |\n               | %{shoes: 59_99, pants: 49_99} |             | true        | Spent $100+      |\n               | %{socks: 10_99}               |             | true        | Socks ship free  |\n               | %{pants: 1_99}                | \"FREE_SHIP\" | true        | Correct coupon   |\n               \"\"\",\n               %{\n                 spending_by_category: spending_by_category,\n                 coupon: coupon,\n                 ships_free?: ships_free?\n               } do\n      shipping_cost = ShippingCalculator.calculate_shipping(spending_by_category, coupon)\n      free_shipping? = shipping_cost == 0\n      assert free_shipping? == ships_free?\n    end\n\nAlternatively, if you don't like the Markdown table format, you can supply a\nhand-rolled list of parameters to the `param_test` macro, like this:\n\n    param_test \"shipping policy matches the web site\",\n                [\n                  # Items in the parameters list can be either maps...\n                  %{spending_by_category: %{pants: 29_99}, coupon: \"FREE_SHIP\"},\n                  # ...or keyword lists\n                  [spending_by_category: %{shoes: 19_99, pants: 29_99}, coupon: nil]\n                ],\n                %{spending_by_category: spending_by_category, coupon: coupon} do\n      ...\n    end\n\nJust make sure that each item in the parameters list has the same keys.\n\nThe final option is to pass a path to a *file* that contains your test parameters (we currently support `.md`/`.markdown`, `.csv`, and `.tsv` files), like this:\n\n```elixir\nparam_test \"pull test parameters from a file\",\n            \"test/fixtures/params.md\",\n            %{\n              spending_by_category: spending_by_category,\n              coupon: coupon,\n              gets_free_shipping?: gets_free_shipping?\n            } do\n  ...\nend\n```","ref":"ParameterizedTest.html"},{"type":"module","title":"Why parameterized testing? - ParameterizedTest","doc":"Parameterized testing reduces toil associated with writing tests that cover\na wide variety of different example cases. It also localizes the test logic\ninto a single place, so that at a glance you can see how a number of\ndifferent factors affect the behavior of the system under test.\n\nAs a bonus, a table of examples (with their expected results) often\nmatches how the business communicates the requirements of a system,\nboth internally and to customers—for instance, in a table describing\nshipping costs based on how much a customer spends, where they're\nlocated, whether they've bought a promotional product, etc. This means\nparameterized tests can often be initially created by pulling directly from\na requirements document that your product folks provide, and the\nproduct folks can later read the tests (or at least the parameters table)\nif they want to verify the behavior of the system.\n\nSee the README for more information.","ref":"ParameterizedTest.html#module-why-parameterized-testing"},{"type":"macro","title":"ParameterizedTest.param_feature/4","doc":"Defines Wallaby feature tests that use your parameters or example data.\n\nThis is to the Wallaby `feature` macro as `param_test` is to `test`.\n\nUse it like this:\n\n    param_feature \"supports Wallaby tests\",\n                  \"\"\"\n                  | text     | url                  |\n                  |----------|----------------------|\n                  | \"GitHub\" | \"https://github.com\" |\n                  | \"Google\" | \"https://google.com\" |\n                  \"\"\",\n                  %{session: session, text: text, url: url} do\n      session\n      |> visit(url)\n      |> assert_has(Wallaby.Query.text(text, minimum: 1))\n    end","ref":"ParameterizedTest.html#param_feature/4"},{"type":"macro","title":"ParameterizedTest.param_test/4","doc":"Defines tests that use your parameters or example data.\n\nUse it like:\n\n    param_test \"grants free shipping for spending $99+ or with coupon FREE_SHIP\",\n               \"\"\"\n               | total_cents | ships_free? | description                 |\n               | ----------- | ----------- | --------------------------- |\n               | 98_99       | false       | Spent too little            |\n               | 99_00       | true        | Min for free shipping       |\n               | 99_01       | true        | Spent more than the minimum |\n               \"\"\",\n               %{total_cents: total_cents, ships_free?: ships_free?} do\n      shipping_cost = ShippingCalculator.calculate(total_cents)\n\n      if ships_free? do\n        assert shipping_cost == 0\n      else\n        assert shipping_cost > 0\n      end\n    end","ref":"ParameterizedTest.html#param_test/4"},{"type":"module","title":"ParameterizedTest.CsvParser","doc":"","ref":"ParameterizedTest.CsvParser.html"},{"type":"function","title":"ParameterizedTest.CsvParser.dump_to_iodata/1","doc":"","ref":"ParameterizedTest.CsvParser.html#dump_to_iodata/1"},{"type":"function","title":"ParameterizedTest.CsvParser.dump_to_stream/1","doc":"","ref":"ParameterizedTest.CsvParser.html#dump_to_stream/1"},{"type":"function","title":"ParameterizedTest.CsvParser.parse_enumerable/2","doc":"","ref":"ParameterizedTest.CsvParser.html#parse_enumerable/2"},{"type":"function","title":"ParameterizedTest.CsvParser.parse_stream/2","doc":"","ref":"ParameterizedTest.CsvParser.html#parse_stream/2"},{"type":"function","title":"ParameterizedTest.CsvParser.parse_string/2","doc":"","ref":"ParameterizedTest.CsvParser.html#parse_string/2"},{"type":"function","title":"ParameterizedTest.CsvParser.to_line_stream/1","doc":"","ref":"ParameterizedTest.CsvParser.html#to_line_stream/1"},{"type":"module","title":"ParameterizedTest.Sigil","doc":"Provides a sigil to wrap parsing of the example test tables.\n\nThe `param_test` macro automatically parses Markdown-style example tables\ninto a list of maps for use in the test contexts, so this sigil is never\n*required* to be used. However, it may occasionally be useful to do things\nlike declare a module attribute which is the pre-parsed example table, or to\ninspect how the table will be parsed.\n\nNote that on Elixir v1.14 and earlier, the sigil is `~x` since those versions\ndon't support multi-character or uppercase sigils.","ref":"ParameterizedTest.Sigil.html"},{"type":"macro","title":"ParameterizedTest.Sigil.sigil_PARAMS/2","doc":"Provides a sigil for producing example data that you can use in tests.","ref":"ParameterizedTest.Sigil.html#sigil_PARAMS/2"},{"type":"macro","title":"Examples - ParameterizedTest.Sigil.sigil_PARAMS/2","doc":"You can have an arbitrary number of columns and rows. Headers are parsed\nas atoms, while the individual cells are parsed as Elixir values.\n\n    iex> ~PARAMS\"\"\"\n    ...>   | plan      | user_permission | can_invite?     |\n    ...>   | :free     | :admin          | true            |\n    ...>   | :free     | :editor         | \"maybe\"         |\n    ...>   | :free     | :view_only      | false           |\n    ...>   | :standard | :admin          | true            |\n    ...>   | :standard | :editor         | \"tuesdays only\" |\n    ...>   | :standard | :view_only      | false           |\n    ...> \"\"\"\n    [\n      {%{plan: :free, user_permission: :admin, can_invite?: true}, _context},\n      {%{plan: :free, user_permission: :editor, can_invite?: \"maybe\"}, _context},\n      {%{plan: :free, user_permission: :view_only, can_invite?: false}, _context},\n      {%{plan: :standard, user_permission: :admin, can_invite?: true}, _context},\n      {%{plan: :standard, user_permission: :editor, can_invite?: \"tuesdays only\"}, _context},\n      {%{plan: :standard, user_permission: :view_only, can_invite?: false}, _context}\n    ]\n\nYou can optionally include separators between the headers and the data.\n\n    iex> ~PARAMS\"\"\"\n    ...>   | plan      | user_permission | can_invite?     |\n    ...>   |-----------|-----------------|-----------------|\n    ...>   | :free     | :admin          | true            |\n    ...>   | :free     | :editor         | \"maybe\"         |\n    ...> \"\"\"\n    [\n      {%{plan: :free, user_permission: :admin, can_invite?: true}, _context},\n      {%{plan: :free, user_permission: :editor, can_invite?: \"maybe\"}, _context}\n    ]\n\nYou can pass the output of `~PARAMS` directly to the `param_test` macro:\n\n    param_test \"distinguishes even and odd numbers\",\n               ~PARAMS\"\"\"\n               | even  | odd |\n               | 2     | 1   |\n               | 4     | 3   |\n               | 6     | 5   |\n               \"\"\",\n               %{even: even, odd: odd} do\n      assert rem(even, 2) == 0\n      assert rem(odd, 2) == 1\n    end","ref":"ParameterizedTest.Sigil.html#sigil_PARAMS/2-examples"},{"type":"module","title":"ParameterizedTest.TsvParser","doc":"","ref":"ParameterizedTest.TsvParser.html"},{"type":"function","title":"ParameterizedTest.TsvParser.dump_to_iodata/1","doc":"","ref":"ParameterizedTest.TsvParser.html#dump_to_iodata/1"},{"type":"function","title":"ParameterizedTest.TsvParser.dump_to_stream/1","doc":"","ref":"ParameterizedTest.TsvParser.html#dump_to_stream/1"},{"type":"function","title":"ParameterizedTest.TsvParser.parse_enumerable/2","doc":"","ref":"ParameterizedTest.TsvParser.html#parse_enumerable/2"},{"type":"function","title":"ParameterizedTest.TsvParser.parse_stream/2","doc":"","ref":"ParameterizedTest.TsvParser.html#parse_stream/2"},{"type":"function","title":"ParameterizedTest.TsvParser.parse_string/2","doc":"","ref":"ParameterizedTest.TsvParser.html#parse_string/2"},{"type":"function","title":"ParameterizedTest.TsvParser.to_line_stream/1","doc":"","ref":"ParameterizedTest.TsvParser.html#to_line_stream/1"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"New feature, and potentially breaking change: [Add the failing parameter line to the backtrace when a test fails](https://github.com/s3cur3/parameterized_test/pull/41) - Changelog","doc":"This change lets each `param_test`/`param_feature` carry forward the context of which row in your parameters table it's executing. Then, when the test fails, it will add that line of the file to the backtrace printed by ExUnit, along with the text of the row.\n\nFor instance, consider this test that will fail 100% of the time:\n\n```elixir\nparam_test \"gives the failing parameter row when a test fails\",\n            \"\"\"\n            | should_fail? | description |\n            | false        | \"Works\"     |\n            | true         | \"Breaks\"    |\n            \"\"\",\n            %{should_fail?: should_fail?} do\n  refute should_fail?\nend\n```\n\nWhen you run this, you'll get an ExUnit backtrace that looks like this:\n\n```\n  1) test gives the failing parameter row when a test fails - Breaks (ParameterizedTest.BacktraceTest)\n     test/parameterized_test/backtrace_test.exs:1\n     Expected truthy, got false\n     code: assert not should_fail?\n     stacktrace:\n       test/parameterized_test/backtrace_test.exs:8: (test)\n       test/parameterized_test/backtrace_test.exs:5: ParameterizedTest.BacktraceTest.\"| true         | \\\"Breaks\\\"    |\"/0\n```\n\nThis should make it easier to figure out which of your parameter rows caused the failing test.\n\nThis is a breaking change if and only if you were using the included sigil (`~PARAMS` for Elixir v1.15+, or `~x` for Elixir v1.14).","ref":"changelog.html#new-feature-and-potentially-breaking-change-add-the-failing-parameter-line-to-the-backtrace-when-a-test-fails-https-github-com-s3cur3-parameterized_test-pull-41"},{"type":"extras","title":"Important bug fix: [Support `@tag` module attributes applied to `param_test` or `param_feature` blocks](https://github.com/s3cur3/parameterized_test/pull/40) - Changelog","doc":"This bug has been there from the beginning, and was making it so that `@tag`s you thought were applying to all the parameterized tests in a block were in fact only applying to the first set of parameters.","ref":"changelog.html#important-bug-fix-support-tag-module-attributes-applied-to-param_test-or-param_feature-blocks-https-github-com-s3cur3-parameterized_test-pull-40"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"- New Mix formatter plugin for formatting the package's sigils, courtesy of @rschenk (🎉). To enable it, in your `formatter.exs`, add to following:\n    ```elixir\n      plugins: [\n        ParameterizedTest.Formatter,\n      ],\n    ```\nYou can see a brief video demo [on the PR](https://github.com/s3cur3/parameterized_test/pull/32).\n- Fixed an issue (#31) where `mix test --failed` could fail to run previously-failing tests because the way we were adding the parameters to the name (as a map) was not stable across runs. The consequence of this change is that the names of tests missing a description will change from listing parameters as maps to lists.\n    - Example: suppose you previously have a `param_test` called `\"checks equality\"` with parameters `val_1: :a` and `val_b: :b`. It would previously have been given the full name `\"checks equality (%{val_1: :a, val_2: :b})\"` *or* `\"checks equality (%{val_2: :b, val_1: :a})\"`, and which you saw would change between test runs. In this release, it will consistently be given the name `\"checks equality ([val_1: :a, val_2: :b])\"`.","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"- Adds a new `param_feature` macro, which wraps Wallaby's `feature` tests\n  the same way `param_test` wraps ExUnit's `test`.\n\n  (While you _can_ use the plain `param_test` macro in a test module that\n  contains `use Wallaby.Feature`, doing so will break some Wallaby features\n  including screenshot generation on failure.)\n- Moves the `parse_examples/2` function, an implementation detail for the\n  `param_test` macro, into a new private module `ParameterizedTest.Parser`.","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"Bug fix to accept more unquoted strings, including those that have Elixir delimiters in them like quotes, parentheses, etc.","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"New features - Changelog","doc":"#### Support treating otherwise unparsable cells in your parameters table as strings\n\nThis is a quality of life improvement for avoiding needing to add noise to string cells:\n\n```elixir\n  param_test \"supports unquoted strings\",\n             \"\"\"\n             | value   | unquoted string |\n             | 1, 2, 3 | The value is 1  |\n             \"\"\",\n             %{value: value, \"unquoted string\": unquoted} do\n    assert value == \"1, 2, 3\" and unquoted == \"The value is 1\"\n  end\n```\n\n#### Support a \"description\" column that will provide a custom name for each test.\n\nIf you supply a column named `description`, `test_description`, or `test_desc`, we'll use that in the test name rather than simply dumping the values from the row in the test table. This lets you provide more human-friendly descriptions of why the test uses the values it does.\n\nA trivial example (which also takes advantage of the support for unquoted strings):\n\n```elixir\n  param_test \"failing test\",\n             \"\"\"\n             | value | description    |\n             | 1     | The value is 1 |\n             \"\"\",\n             %{value: value} do\n    assert value == 2\n  end\n```\n\nWhen you run this, the error will include the description (\"The value is 1\") in the test name:\n\n```\n  1) test failing test - The value is 1 (MyAppTest)\n     test/my_app_test.exs:8\n     Assertion with == failed\n     code:  assert value == 2\n     left:  1\n     right: 2\n     stacktrace:\n       test/my_app_test.exs:14: (test)\n```\n\nThis is useful for communication with stakeholders, or for understanding what went wrong when a test fails.","ref":"changelog.html#new-features"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"There are two new features in this release thanks to new contributor @axelson:\n\n* [Support longer test names](https://github.com/s3cur3/parameterized_test/pull/17)\n* [Support comments and Obsidian markdown table format](https://github.com/s3cur3/parameterized_test/pull/16)","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"- Renamed to `ParameterizedTest`, with the accompanying macro `param_test`.\n    (Why not `parameterized_test`? It's longer, harder to spell, and there are a lot of\n    other accepted spellings, including \"parameterised,\" \"parametrized,\" and \"parametrised.\")\n- Added support for hand-rolled lists of parameters, like:\n\n    ```elixir\n    param_test \"shipping policy matches the web site\",\n                [\n                  # Items in the parameters list can be either maps...\n                  %{spending_by_category: %{pants: 29_99}, coupon:   \"FREE_SHIP\"},\n                  # ...or keyword lists\n                  [spending_by_category: %{shoes: 19_99, pants: 29_99}, coupon: nil]\n                ],\n                %{spending_by_category: spending_by_category, coupon: coupon} do\n    ...\n    end\n    ```\n- Added experimental support for populating test parameters from CSV and TSV files.\n  Eventually I'd like to extend this to other sources like Notion documents. \n  (Feedback welcome—just open an issue!)","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"v0.0.1 - Changelog","doc":"Initial release.","ref":"changelog.html#v0-0-1"},{"type":"extras","title":"ParameterizedTest","doc":"# ParameterizedTest\n\n[![Hex.pm](https://img.shields.io/hexpm/v/parameterized_test)](https://hex.pm/packages/parameterized_test) [![Build and Test](https://github.com/s3cur3/parameterized_test/actions/workflows/elixir-build-and-test.yml/badge.svg)](https://github.com/s3cur3/parameterized_test/actions/workflows/elixir-build-and-test.yml) [![Elixir Quality Checks](https://github.com/s3cur3/parameterized_test/actions/workflows/elixir-quality-checks.yml/badge.svg)](https://github.com/s3cur3/parameterized_test/actions/workflows/elixir-quality-checks.yml) [![Code coverage](https://codecov.io/gh/s3cur3/parameterized_test/graph/badge.svg)](https://codecov.io/gh/s3cur3/parameterized_test)\n\nA utility for defining eminently readable parameterized (or example-based) tests in \nElixir's ExUnit, inspired by [example tests in Cucumber](https://cucumber.io/docs/guides/10-minute-tutorial/?lang=java#using-variables-and-examples).","ref":"readme.html"},{"type":"extras","title":"What are parameterized tests? - ParameterizedTest","doc":"Parameterized tests let you define variables along a number of dimensions \nand re-run the same test body (including all `setup`) for each \ncombination of variables.\n\nA simple example:\n\n```elixir\nsetup context do\n  # context.permissions gets set by the param_test below\n  permissions = Map.get(context, :permissions, nil)\n  user = AccountsFixtures.user_fixture(permissions: permissions)\n  %{user: user}\nend\n\nparam_test \"users with editor permissions or better can edit posts\",\n           \"\"\"\n           | permissions | can_edit? | description                     |\n           |-------------|-----------|---------------------------------|\n           | :admin      | true      | Admins have max permissions     |\n           | :editor     | true      | Editors can edit (of course!)   |\n           | :viewer     | false     | Viewers are read-only           |\n           | nil         | false     | Anonymous viewers are read-only |\n           \"\"\",\n           %{user: user, permissions: permissions, can_edit?: can_edit?} do\n  assert Posts.can_edit?(user) == can_edit?, \"#{permissions} permissions should grant edit rights\"\nend\n```\n\nThat test will run 4 times, with the variables from from the table being applied \nto the test's context each time (and therefore being made\navailable to the `setup` handler). These variables are:\n\n- `:permissions` \n- `:can_edit?`\n- the special `:description` variable (see\n  [About test names, and improving debuggability](#about-test-names-and-improving-debuggability)\n  for how this is used)\n\nThus, under the hood this generates four unique tests,\nequivalent to doing something like this:\n\n```elixir\nsetup context do\n  permissions = Map.get(context, :permissions, nil)\n  user = AccountsFixtures.user_fixture{permissions: permissions}\n  %{user: user}\nend\n\nfor {permissions, can_edit?, description} <- [\n        {:admin,  true,  \"Admins have max permissions\"},\n        {:editor, true,  \"Editors can edit (of course!)\"},\n        {:viewer, false, \"Viewers are read-only\"},\n        {nil,     false, \"Anonymous viewers are read-only\"}\n      ] do\n    @permissions permissions\n    @can_edit? can_edit?\n    @description description\n\n    @tag permissions: @permissions\n    @tag can_edit?: @can_edit?\n    @tag description: @description\n    test \"users with at least editor permissions can edit posts — #{@description}\", %{user: user} do\n      assert Posts.can_edit?(user) == @can_edit?\n    end\n  end\nend\n```\n\nAs you can see, even with only 3 variables (just 2 that impact the test semantics!),\nthe `for` comprehension comes with a lot of boilerplate. But the `param_test`\nmacro supports an arbitrary number of variables, so you can describe complex\nbusiness rules like \"users get free shipping if they spend more than $100,\nor if they buy socks, or if they have the right coupon code\":\n\n```elixir\nparam_test \"grants free shipping based on the marketing site's stated policy\",\n            \"\"\"\n            | spending_by_category          | coupon      | ships_free? | description      |\n            |-------------------------------|-------------|-------------|------------------|\n            | %{shoes: 19_99, pants: 29_99} |             | false       | Spent too little |\n            | %{shoes: 59_99, pants: 49_99} |             | true        | Spent over $100  |\n            | %{socks: 10_99}               |             | true        | Socks ship free  |\n            | %{pants: 1_99}                | \"FREE_SHIP\" | true        | Correct coupon   |\n            | %{pants: 1_99}                | \"FOO\"       | false       | Incorrect coupon |\n            \"\"\",\n            %{\n               spending_by_category: spending_by_category,\n               coupon: coupon,\n               ships_free?: ships_free?\n             } do\n  shipping_cost = ShippingCalculator.calculate(spending_by_category, coupon)\n  \n  if ships_free? do\n    assert shipping_cost == 0\n  else\n    assert shipping_cost > 0\n  end\nend\n```\n\nThe package also provides a second macro, `param_feature`, which wraps\nWallaby's `feature` tests the same way `param_test` wraps ExUnit's `test`.\n(While you _can_ use the plain `param_test` macro in a test module that\ncontains `use Wallaby.Feature`, doing so will break some Wallaby features\nincluding screenshot generation on failure.)","ref":"readme.html#what-are-parameterized-tests"},{"type":"extras","title":"Why parameterized testing? - ParameterizedTest","doc":"Parameterized testing reduces toil associated with writing tests that cover\na wide variety of different example cases. It also localizes the test logic\ninto a single place, so that at a glance you can see how a number of\ndifferent factors affect the behavior of the system under test.\n\nAs a bonus, a table of examples (with their expected results) often\nmatches how the business communicates the requirements of a system,\nboth internally and to customers—for instance, in a table describing\nshipping costs based on how much a customer spends, where they're\nlocated, whether they've bought a promotional product, etc. This means\nparameterized tests can often be initially created by pulling directly from\na requirements document that your product folks provide, and the\nproduct folks can later read the tests (or at least the parameters table)\nif they want to verify the behavior of the system.","ref":"readme.html#why-parameterized-testing"},{"type":"extras","title":"Parameterized tests versus property tests - ParameterizedTest","doc":"Parameterized tests are superficially similar to property-based tests.\nBoth allow you to write fewer tests while covering more of your system's\nbehavior. This library is not a replacement for property tests, but\nrather complimentary to them.\n\nThere are a few reasons you might choose to write a parameterized\ntest rather than a property test:\n\n- **When describing policies, not invariants**: Much of a system's\n  business logic comes down to arbitrary choices made by a product team.\n  For instance, there's nothing in the abstract description of a shipping\n  calculator that says buying socks or spending $100 total should grant\n  you free shipping. Those aren't *principles* that every correctly\n  implemented shipping system would implement. Instead, they're choices\n  made by someone (maybe a product manager) which will in all likelihood\n  be fiddled with over time.\n  \n  Contrast that with the classic use cases for property tests: every \n  (correct) implementation of, say, `List.sort/1` will *always* have\n  the dual properties of:\n  \n  1. every element of the input being represented in the output, and \n  2. every element being \"less than\" the element after it.\n\n  These sorting properties are *invariants* of the sorting function,\n  and therefore are quite amenable to property testing.\n- **Ease of writing**: Property tests take a lot of practice to get\n  good at writing. They're often quite time consuming to produce, and\n  even when you think you've adequately described the parameters to\n  the system.\n- **For communication with other stakeholders**: The table of examples\n  in a parameterized test can be made readable by non-programmers (or \n  non-Elixir  programmers), so they can be a good way of showing others\n  in your organization which behaviors of the system you've verified.\n  Because they can compactly express a lot of test cases, they're\n  much more suitable for this than saying \"go read the title of\n  every line in this file that starts with `test`.\"\n- **For verifying the exact scenarios described by other stakeholders**:\n  Sometimes the edges of a particular behavior may be fuzzy—not just\n  to you, but in the business domain as well. Hammering out hard-and-fast\n  rules may not be necessary or worth it, so property tests that exercise\n  the boundaries would be overkill. In contrast, when your product\n  folks produce a document that describes the behavior of particular\n  scenarios, you can encode that in a table and ensure that for the\n  cases that are well-specified, the system behaves correctly.\n\nWhen would you write a property test instead of an example tests?\n\n- When you can specify true invariants about the desired behavior\n- When you want the absolute highest confidence in your code\n- When the correctness of a piece of code is important enough to merit\n  a large time investment in getting the tests right\n- When the system's behavior at the edges is well specified\n\nAnd of course there's nothing wrong with using a mix of normal tests,\nparameterized tests, and property tests for a given piece of functionality.","ref":"readme.html#parameterized-tests-versus-property-tests"},{"type":"extras","title":"Installation and writing your first test - ParameterizedTest","doc":"1. Add `parameterized_test` to your `mix.exs` dependencies:\n\n    ```elixir\n    def deps do\n      [\n        {:parameterized_test, \"~> 0.6\", only: [:test]},\n      ]\n    end\n    ```\n2. Run `$ mix deps.get` to download the package\n3. Write your first example test by adding `import ParameterizedTest` \n   to the top of your test module, and using the `param_test` macro.\n\n   You can optionally include a separator between the header and body\n   of the table (like `|--------|-------|`), and a `description` column\n   to improve the errors you get when your test fails (see\n   [About test names, and improving debuggability](#about-test-names-and-improving-debuggability)\n   for more on descriptions).\n\n   The header of your table will be parsed as atoms to pass into your\n   test context. The body cells of the table can be any valid Elixir \n   expression, and empty cells will produce a `nil` value.\n\n   A dummy example:\n\n    ```elixir\n    defmodule MyApp.MyModuleTest do\n      use ExUnit.Case, async: true\n      import ParameterizedTest\n\n      param_test \"behaves as expected\",\n                 \"\"\"\n                 | variable_1       | variable_2           | etc     |\n                 | ---------------- | -------------------- | ------- |\n                 | %{foo: :bar}     | div(19, 3)           | false   |\n                 | \"bip bop\"        | String.upcase(\"foo\") | true    |\n                 | [\"whiz\", \"bang\"] | :ok                  |         |\n                 |                  | nil                  | \"maybe\" |\n                 \"\"\",\n                 %{\n                   variable_1: variable_1,\n                   variable_2: variable_2,\n                   etc: etc\n                 } do\n          assert MyModule.valid_combination?(variable_1, variable_2, etc)\n        end\n    end\n    ```","ref":"readme.html#installation-and-writing-your-first-test"},{"type":"extras","title":"Debugging failing tests - ParameterizedTest","doc":"As of v0.6.0, when you get a test failure, the backtrace that ExUnit prints will include\nthe line in your file that provided the failing parameters. For instance, consider\nthis test that will fail 100% of the time:\n\n```elixir\nparam_test \"gives the failing parameter row when a test fails\",\n            \"\"\"\n            | should_fail? | description |\n            | false        | \"Works\"     |\n            | true         | \"Breaks\"    |\n            \"\"\",\n            %{should_fail?: should_fail?} do\n  refute should_fail?\nend\n```\n\nWhen you run it, you'll get an ExUnit backtrace that looks like this:\n\n```\n  1) test gives the failing parameter row when a test fails - Breaks (ParameterizedTest.BacktraceTest)\n     test/parameterized_test/backtrace_test.exs:1\n     Expected truthy, got false\n     code: assert not should_fail?\n     stacktrace:\n       test/parameterized_test/backtrace_test.exs:8: (test)\n       test/parameterized_test/backtrace_test.exs:5: ParameterizedTest.BacktraceTest.\"| true         | \\\"Breaks\\\"    |\"/0\n```\n\nUse this line number to figure out which of your parameter rows caused the failing test.","ref":"readme.html#debugging-failing-tests"},{"type":"extras","title":"About test names, and improving debuggability - ParameterizedTest","doc":"ExUnit requires each test in a module to have a unique name. By default,\nwithout a `description` for the rows in your parameters table, \n`ParameterizedTest` appends a stringified version of the parameters\npassed to your test to the name you give the test. Consider this test:\n\n```elixir\nparam_test \"checks equality\",\n            \"\"\"\n            | val_1 | val_2 |\n            | :a    | :a    |\n            | :b    | :c    |\n            \"\"\",\n            %{val_1: val_1, val_2: val_2} do\n  assert val_1 == val_2\nend\n```\n\nUnder the hood, this produces two tests with the names:\n\n- `\"checks equality ([val_1: :a, val_b: :a])\"`\n- `\"checks equality ([val_1: :b, val_b: :c])\"`\n\nAnd if you ran this test, you'd get an error that looks like this:\n\n```\n  1) test checks equality ([val_1: :b, val_2: :c]) (MyModuleTest)\n     test/my_module_test.exs:4\n     Assertion with == failed\n     code:  assert val_1 == val_2\n     left:  :b\n     right: :c\n     stacktrace:\n       test/my_module_test.exs:11: (test)\n```\n\nYou can improve the names in the failure cases by providing a `description`\ncolumn. When provided, that column will be used in the name. You may want\nto use this to explain *why* this combination of values should produce\nthe expected outcome; for instance:\n\n```elixir\nparam_test \"grants free shipping for spending $99 or more, or with coupon FREE_SHIP\",\n           \"\"\"\n           | total_cents | coupon      | free? | description                 |\n           | ----------- | ----------- | ----- | --------------------------- |\n           | 98_99       |             | false | Spent too little            |\n           | 99_00       |             | true  | Min for free shipping       |\n           | 99_01       |             | true  | Spent more than the minimum |\n           | 1_00        | \"FREE_SHIP\" | true  | Had the right coupon        |\n           | 1_00        | \"FOO\"       | false | Unrecognized coupon         |\n           \"\"\", %{total_cents: total_cents, coupon: coupon, free?: gets_free_shipping?} do\n  shipping_cost = ShippingCalculator.calculate(total_cents, coupon)\n  free_shipping? = shipping_cost == 0\n  assert free_shipping? == gets_free_shipping?\nend\n```\n\nSuppose in your `ShippingCalculator` implementation, you mistakenly set \nthe free shipping threshold to be _greater_ than $99.00, when your web site's\nstate policy was $99 or more. You'd get an error when running this test that\nlooks like this (note the first line ends with \"Spent the min. for free shipping\" from the `description` column):\n\n```\n  1) test grants free shipping for spending $99 or more, or with coupon FREE_SHIP - Spent the min. for free shipping (ShippingCalculatorTest)\n     test/shipping/shipping_calculator_test.exs:34\n     Assertion with == failed\n     code:  assert free_shipping? == gets_free_shipping?\n     left:  false\n     right: true\n     stacktrace:\n       test/shipping/shipping_calculator_test.exs:47: (test)\n```","ref":"readme.html#about-test-names-and-improving-debuggability"},{"type":"extras","title":"Objections - ParameterizedTest","doc":"","ref":"readme.html#objections"},{"type":"extras","title":"Why not just use the new `:parameterize` feature built into ExUnit in Elixir 1.18? - ParameterizedTest","doc":"Both this package and the\n[new, built-in parameterization](https://hexdocs.pm/ex_unit/main/ExUnit.Case.html#module-parameterized-tests)\ndo similar things: they re-run the same test body with a different set of\nparameters. However, the built-in parameterization works by re-running \nyour entire test module with each parameter set, so it's primarily aimed\nat cases where the tests should work the same regardless of the parameters.\n(The docs give the example of testing the `Registry` module and expecting\nit to behave the same regardless of how many partitions are used.)\n\nIn contrast, the `param_test` macro is designed to use different parameters\non a per-test basis, and it's expected that the parameters will cause different\nbehavior between the test runs (and you'd generally expect to see one column\nthat describes what the results should be).\n\nFinally, of course, there's the format of a `param_test`. The tabular, often\nquite human-friendly format encourages collaboration with less technical\npeople on your team; your product manager may not be able to read a `for`\ncomprehension, but if you link them to a Markdown table on GitHub that shows\nthe test cases you've covered, they can probably make sense of them.\n\n### \"I hate the Markdown table syntax!\"\n\nNo sweat, you don't have to use it. You can instead pass a hand-rolled list of\nparameters to the `param_test` macro, like this:\n\n```elixir\nparam_test \"shipping policy matches the web site\",\n            [\n              # Items in the parameters list can be either maps...\n              %{spending_by_category: %{pants: 29_99}, coupon: \"FREE_SHIP\"},\n              # ...or keyword lists\n              [spending_by_category: %{shoes: 19_99, pants: 29_99}, coupon: nil]\n            ],\n            %{spending_by_category: spending_by_category, coupon: coupon} do\n  ...\nend\n```\n\nJust make sure that each item in the parameters list has the same keys.\n\nThe final option is to pass a path to a *file* that contains your test parameters (we currently support `.md`/`.markdown`, `.csv`, and `.tsv` files), like this:\n\n```elixir\nparam_test \"pull test parameters from a file\",\n            \"test/fixtures/params.md\",\n            %{\n              spending_by_category: spending_by_category,\n              coupon: coupon,\n              gets_free_shipping?: gets_free_shipping?\n            } do\n  ...\nend\n```","ref":"readme.html#why-not-just-use-the-new-parameterize-feature-built-into-exunit-in-elixir-1-18"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}